//Add support for space functions myFunc(3)  -->  myFunc 3 LATER
//Creates a tree using the divisions from the lexer
function parse(array) {
	array = structurize(array)
	var tree = divide(array)
	tree = parseTree(tree)
	return tree;
}

function structurize(array) { //Recursive --> Creates a tree by using the brackets found on the code { } [ ] ( )
	var array2 = []
	brackets = 0 //Number of brackets that we have opened and not yet closed. [ { (
	var fromLine = 0 //Next line to be added
	for (var i = 0; i < array.length; i++) {
		if (array[i].type == "openSquareBracket" || array[i].type == "openCurlyBracket" || array[i].type == "openBracket") {
			if (brackets == 0) { 
				fromLine = i
			}
			brackets++
		}
		if (array[i].type == "closeSquareBracket" || array[i].type == "closeCurlyBracket" || array[i].type == "closeBracket") {
			brackets--
		}
		if (brackets == 0) { //We are not breaking any brackets
			if (array[i].type == "closeSquareBracket" || array[i].type == "closeCurlyBracket" || array[i].type == "closeBracket") {
				var index = array2.length //We want to add it to the end of array2
				array2[index] = {}
				if (array[i].type == "closeSquareBracket") {
					array2[index].type = "squareBrackets"
				}
				if (array[i].type == "closeCurlyBracket") {
					array2[index].type = "curlyBrackets"
				}
				if (array[i].type == "closeBracket") {
					array2[index].type = "brackets"
				}
				//Parse the part inside
				array2[index].text = array.slice(fromLine + 1, i) //+1 is used so that we don't add the brackets too. 
				array2[index].text = structurize(array2[index].text)
				fromLine = i
			} else {
				array2.push(array[i]) //Add it at the end
				fromLine = i + 1
			}
		} else if (brackets < 0) {
			var d
			if (array[i].type == "openSquareBracket") {
				d = "]"
			}
			if (array[i].type == "openCurlyBracket") {
				d = "}"
			}
			if (array[i].type == "openBracket") {
				d = "}"
			}
			throwError(undefined, 0, [d])
		}
	}
	return array2
}

function divide(array) { //Recursive --> Divides the array into instructions using the dividers found on the code \n ;
	var array2 = []
	var fromLine = 0 //Next line to be added
	for (var i = 0; i < array.length; i++) {
		if (array[i].type == "divider") {
			array2.push(array.slice(fromLine, i))
			fromLine = i + 1
		}
		if (array[i].type == "brackets" || array[i].type == "curlyBrackets" || array[i].type == "squareBrackets") { //Parse inside the brackets too
			array[i].text = divide(array[i].text)
		}
	}
	if (fromLine != array.length) {
		array2.push(array.slice(fromLine, i)) //Last part doesn't need a divider
	}
	return array2
}

function parseTree(tree){
	for(var i =0;i<tree.length;i++){
		tree[i] = parseStatement(tree[i])
	}
	return tree
}

function getArrayBack(array,matches,k){ //Gets back the original array from another array of strings. Used in parseStatement()
	var text = matches[k] //The string that has matched this part
	var numbers = text.match(/\d+/g) //Indexes that were found in the string
	for(var l=0;l<numbers.length;l++){
		numbers[l] = parseFloat(numbers[l]) //Convert from string to number (we use parseFloat because it's faster than parseInt)
		numbers[l] = array[numbers[l]] //Get the original node
	}
	return numbers
}

var grammar = [ //List of grammar rules
// = += -= *= /= ^= .=
/^(&variable\d+)(&equality\d+)(.*)$/, // j = 0
// |
/^(.*)(&or\d+)(.*)$/, // j = 1
// &
/^(.*)(&and\d+)(.*)$/, // j = 2
// == !=
/^(.*)(&logicalEquality\d+)(.*)$/, // j = 3
// 1 < a < 5
// < <= > >=
/^(.*)(&relational\d+)(.*)$/, // j = 4
// For - multiple lines
/^&for\d+(&brackets\d+)(&curlyBrackets\d+)$/, // j = 5
// For - inline
/^&for\d+(&brackets\d+)(.*)$/, // j = 6
// Function definition
/^(&variable\d+)(&brackets\d+)(&curlyBrackets\d+)$/, // j = 7
// + -
/^(.+)(&firstOperator\d+)(.+)$/, // j = 8
// * / % ^
/^(.+)(&secondOperator\d+)(.+)$/, // j = 9
// +3 -3
// .
/^(.+)(&point\d+)(&variable\d+)$/, // j = 10
// ++ --
/^(&variable\d+)(&selfOperator\d+)$/, // j = 11
// Function call
/^(&variable\d+)(&brackets\d+)$/, // j = 12
// Object
/^(&curlyBrackets\d+)$/, // j = 13
// Array
/^(&squareBrackets\d+)$/, // j = 14
// Variable
/^(&variable\d+)$/, //j = 15
// Literals
/^(&literal\d+)$/, //j = 16
// Brackets
/^(&brackets\d+)$/ // j = 17
]

function parseStatement(array){
	//Convert the array into a string representation so that we can use a regEx on it
	var stringRepresentation = ""
	for(var j = 0; j<array.length; j++){
		stringRepresentation+="&"+array[j].type+j
	}
	//Divide the array if it has commas too. Before parsing anything
	if(stringRepresentation.match(/&comma\d*/)!=null){
		var newArray = [[]]
		var index = 0
		for(var m = 0;m <array.length;m++){
			if(array[m].type=="comma"){
				index++
				newArray[index] = []
			}
			else{
				newArray[index][newArray[index].length] = array[m]
			}
		}
		for(var m = 0;m<newArray.length;m++){
			newArray[m] = parseStatement(newArray[m])
		}
		array = newArray //Save it
	}
	else{
		for(var j = 0; j<grammar.length;j++){ //Try to match every grammar rule
			var matches = stringRepresentation.match(grammar[j]) //See if it matches the regEx
			if(matches != null){ //The grammar rule matched something
				var newArray = {}
				//Get back the original nodes

				// = += -= *= /= ^= .=    |    &    == !=    < <= > >=    + -    * / % ^ 
				if(j == 0 | j == 1 || j == 2 || j == 3 || j == 4 || j == 8 | j == 9 | j == 10){
					newArray[0] = getArrayBack(array,matches,1) //Before sign
					newArray.type = getArrayBack(array,matches,2)[0].text // Sign
					newArray[1] = getArrayBack(array,matches,3) //After sign
					//Parse them too
					newArray[0] = parseStatement(newArray[0])
					newArray[1] = parseStatement(newArray[1])
				}

				// For - multiple lines
				if (j == 5) {
					newArray.initialization = getArrayBack(array,matches,1)[0].text[0]
					newArray.condition = getArrayBack(array,matches,1)[0].text[1]
					newArray.afterthought = getArrayBack(array,matches,1)[0].text[2]
					newArray.type = "for"
					newArray[0] = getArrayBack(array,matches,2)[0].text //Loop statements
					//Parse them too
					newArray.initialization = parseStatement(newArray.initialization)
					newArray.condition = parseStatement(newArray.condition)
					newArray.afterthought = parseStatement(newArray.afterthought)
					//Parse its children too
					for(var p =0; p<newArray[0].length;p++){
						newArray[0][p] = parseStatement(newArray[0][p])
					}
				}

				// For - inline
				if (j == 6) {
					newArray.initialization = getArrayBack(array,matches,1)[0].text[0]
					newArray.condition = getArrayBack(array,matches,1)[0].text[1]
					newArray.afterthought = getArrayBack(array,matches,1)[0].text[2]
					newArray.type = "for"
					newArray[0] = getArrayBack(array,matches,2) //Loop statements
					//Parse them too
					newArray.initialization = parseStatement(newArray.initialization)
					newArray.condition = parseStatement(newArray.condition)
					newArray.afterthought = parseStatement(newArray.afterthought)
					newArray[0] = parseStatement(newArray[0])
				}

				// Function definition
				if (j == 7) { 
					newArray.text = getArrayBack(array,matches,1)[0].text //Function name
					newArray.type = "funcDefinition"
					newArray.parameters = getArrayBack(array,matches,2) //Function parameters
					newArray[0] = getArrayBack(array,matches,3)[0].text //Function statements
					//Parse them too
					newArray.parameters = parseStatement(newArray.parameters)
					for(var p =0; p<newArray[0].length;p++){
						newArray[0][p] = parseStatement(newArray[0][p])
					}
				}

				// Self operator
				if (j == 11) { 
					newArray[0] = getArrayBack(array,matches,1)[0]
					newArray.type = getArrayBack(array,matches,2)[0].text
					//Parse it too
					newArray[0] = parseStatement(newArray[0])
				}

				// Function call
				if (j == 12) { 
					newArray.text = getArrayBack(array,matches,1)[0].text //Function name
					newArray.type = "funcCall"
					newArray.parameters = getArrayBack(array,matches,2)[0].text[0] //Function parameters
					//Parse it too
					newArray.parameters = parseStatement(newArray.parameters)
				}

				// Object
				if (j == 13) { 
					newArray.children = getArrayBack(array,matches,1)[0].text
					newArray.type = "object"
					//Parse its children too
					for(var p =0; p<newArray.children.length;p++){
						newArray.children[p] = parseStatement(newArray.children[p])
					}
				}

				// Array
				if (j == 14) { 
					newArray.children = getArrayBack(array,matches,1)[0].text[0]
					newArray.type = "array"
					//Parse it too
					newArray.children = parseStatement(newArray.children)
				}

				// Variable & literal
				if (j == 15 || j == 16) { 
					newArray.text = getArrayBack(array,matches,1)[0].text
					if(j == 15){
						newArray.type = "variable"
					}
					else{
						newArray.type = "literal"
					}
				}

				if (j == 17) { 
					newArray = getArrayBack(array,matches,1)[0].text[0]
					newArray = parseStatement(newArray)
				}

				array = newArray //Save it
				break;
			}
			else if(j==grammar.length-1){
				if(stringRepresentation!=""){
					console.log(array)
					console.log(stringRepresentation) //Wasn't parsed
				}
			}
		}
	}
	return array
}